#include <SPI.h>
#define SS 10 // ArduinoUNO 10 : M5Stamp 19 : RaspberryPi PICO 5

uint32_t pressure = 0 ;
float temperature = 0 ;

// Register読み取り
uint8_t readLPS(uint8_t addr) {
  uint8_t readData ;

  digitalWrite(SS, HIGH);
  digitalWrite(SS, LOW);  //通信開始
  SPI.transfer(addr|0x80);
  readData = SPI.transfer(0x00);
  digitalWrite(SS, HIGH); //通信終了  
  return readData ;
}

// 1バイトコマンドを送信
void sendLPS(uint8_t addr , uint8_t command) {
  digitalWrite(SS, HIGH);
  digitalWrite(SS, LOW);  //通信開始
  SPI.transfer(addr);
  SPI.transfer(command);
  digitalWrite(SS, HIGH); //通信終了  
}

// 気圧センサー読み取り
bool readLPS_Sencer( ) {
  uint8_t stat = readLPS(0x27);
  uint8_t pxl = readLPS(0x28) ;
  uint8_t pl = readLPS(0x29) ;
  uint8_t ph = readLPS(0x2A) ;
  uint8_t tl = readLPS(0x2B) ;
  uint8_t th = readLPS(0x2C) ;
  uint8_t fifoStat = readLPS(0x2F);

  // combine bytes
  pressure = (int32_t)(int8_t)ph << 16 | (uint16_t)pl << 8 | pxl;
  pressure /= 4096 ;
  int16_t temp = (int16_t)th << 8 | tl ;
  temperature = 42.5 + (float)temp / 480;

  Serial.print("stat = ") ;
  Serial.print(stat,HEX) ;
  Serial.print("  fifoStat = ") ;
  Serial.print(fifoStat,HEX) ;
  Serial.print("  ") ;
  
  return ((stat & 0x03) == 0x03) && ((fifoStat & 0xC0) == 0xC0)  ;
}

void setup() {
  Serial.begin(9600);
  while (!Serial); // Leonardo: wait for serial monitor
  Serial.println("\nLPS25HB 気圧センサーテスト SPI/FIFO有効");

  pinMode (SS, OUTPUT);
  digitalWrite(SS, HIGH);
  SPI.begin ();
  SPI.setBitOrder(MSBFIRST);
  SPI.setDataMode(SPI_MODE3);
  SPI.setClockDivider(SPI_CLOCK_DIV16);

  // -- Read WHO AM I
  uint8_t deviceID = readLPS(0x0F) ;
  Serial.print("Device ID = ") ;
  Serial.println(deviceID,HEX) ;

  // --- Reset LPS25HB
  sendLPS(0x21,0x04);   // ソフトウェアリセット
  while(readLPS(0x21)); // リセット確認
  delay(100);
  sendLPS(0x21,0x80);   // リブートメモリ
  while(readLPS(0x21)); // リブート確認
  delay(100);
  sendLPS(0x20,0x90);   // アクティブモード、アウトプット25Hz
                        // ※VCCに5V入れた場合、1Hzだと何故か誤差が大きい
                        // ※この設定だと+2hPa +1℃位のズレで収まる
  delay(100);
  sendLPS(0x21,0x40);   // FiFO有効
  sendLPS(0x2E,0xC7);   // FIFO平均モード 8サンプル
                        // ※サンプル数が大きすぎるとFIFOがFULLになるまでに時間がかかる
  delay(1300);
}

void loop() {
  // ----- Read 気圧センサー
  bool ret = readLPS_Sencer( ) ;
  Serial.print("ret = ") ;
  Serial.print(ret) ;
  Serial.print("  /  pressure = ") ;
  Serial.print(pressure) ;
  Serial.print("  /  Temperature = ") ;
  Serial.println(temperature) ;

  delay(10*1000) ; // 10秒ディレイ
}
